/*
 * File:   capboard_bridge.c
 * Author: Raphael Wimmer <raphael@hcilab.org>
 *
 * Shove data from /dev/ttyS0 to a TCP socket
 *
 * Licensed under the GNU GPL v2 or later
 * see http://www.gnu.org/licenses/gpl.html
 *
 * Based on TCP example code by Kien Pham (in Public Domain)
 * from http://xengren.tripod.com/linnet/
 * and
 * the Linux serial programming howto (under LDP license)
 * by Peter Baumann and Gary Frerking
 * at http://tldp.org/HOWTO/Serial-Programming-HOWTO/
 *
 * Created on May 24, 2006, 6:54 PM
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>	// Include these for socket(), connect(), bind(), etc.
#include <sys/types.h>
#include <netdb.h>		// Include this for getprotobyname()
#include <string.h>	// Include this for memset()
#include <netinet/in.h>	// Include this for htonl(), htons(), etc.
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <signal.h>
#include <errno.h>



#define BAUDRATE 115200
#define MODEMDEVICE "/dev/ttyS0"
#define _POSIX_SOURCE 1 /* POSIX compliant source */
#define	PORT	2000  // Port to listen on...

void quit(int sig); // signal handler
void parser(char*);

char	val[200];
char	message[255];	// string to send to the client.
long	sent_lines = 0;


// Variables for the serial port component of the application.
int fd, c, res;
struct termios oldtio, newtio;

// Variables for the TCP socket component of the application.
int file_descriptor;	// File descriptor that represents the server socket.
int	inbound_connection;	// File descriptor that represents the socket of the inbound connection.
struct	sockaddr_in	server_address;	// Really only contains the port we want to listen on.
struct	sockaddr_in	inbound_address;	// Address of the inbound connection.
int	inbound_address_size;	// Size of the structure for the inbound connection.
unsigned	char	*address_holder;	// Pointer to simplify the extraction of IP addresses.
//char	buffer[256];		// Buffer to hold incoming data from the client.

struct values{

	double val;
	
	int type;

};

void parser(char* value)

{
	
	struct values measure;
	
	if(value[0] == 'T' || value[0] == 't' )
	
	{
		
		measure.type = 1;
	
		measure.val = atof(value+2);
		
		printf("%lf\n", measure.val);
		
	}
	
	else if(value[0] == 'H' || value[0] == 'h')
		
	{
		
		measure.type = 2;
		
		measure.val = atof(value+2);
		
		printf("%lf\n", measure.val);
		
	}
	
	else
		
	{
		
		printf("Wrong Input\n");
		
	}

}

void init_tty(char* device, int baudrate){
    int speed;
    switch(baudrate){ // damn constants!!!
		case	600:	speed = B600; break;
		case	1200:	speed = B1200; break;
		case	2400:	speed = B2400; break;
		case	4800:	speed = B4800; break;
		case	9600:	speed = B9600; break;
		case	19200:	speed = B19200; break;
		case	38400:	speed = B38400; break;
		case	57600:	speed = B57600; break;
		case	115200:	speed = B115200; break;
		case	230400:	speed = B230400; break;
		default:    printf("Sorry, this baudrate is not supported.\n"); exit(-1);
	    }
    
    
    /*
     Open modem device for reading and writing and not as controlling tty
     because we don't want to get killed if linenoise sends CTRL-C.
     */
    
    
    
    fd = open(device, O_RDWR | O_NOCTTY);
    if (fd <0) 
        {perror(device); exit(-1); }
    else 
        {fcntl(fd, F_SETFL, 0); }

    
    printf("Reading from %s (%d baud)\n", device, baudrate);
    
    tcgetattr(fd, &oldtio); /* save current serial port settings */
    bzero(&newtio, sizeof(newtio)); /* clear struct for new port settings */
    
    /*
     BAUDRATE: Set bps rate. You could also use cfsetispeed and cfsetospeed.
     CRTSCTS : output hardware flow control (only used if the cable has
     all necessary lines. See sect. 7 of Serial-HOWTO)
     CS8     : 8n1 (8bit,no parity,1 stopbit)
     CLOCAL  : local connection, no modem contol
     CREAD   : enable receiving characters
     */
    //newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
    newtio.c_cflag |= (CLOCAL | CREAD);


    newtio.c_cflag &= ~PARENB;
    newtio.c_cflag &= ~CSTOPB;
    newtio.c_cflag &= ~CSIZE;
    newtio.c_cflag |= CS8;



    cfsetispeed(&newtio, speed);
    cfsetospeed(&newtio, speed);
    /*
     IGNPAR  : ignore bytes with parity errors
     ICRNL   : map CR to NL (otherwise a CR input on the other computer
     will not terminate input)
     otherwise make device raw (no other input processing)
     */
    newtio.c_iflag = IGNPAR | ICRNL;
    
    /*
     Raw output.
     */
    newtio.c_oflag = 0;
    
    /*
     ICANON  : enable canonical input
     disable all echo functionality, and don't send signals to calling program
     */
    newtio.c_lflag = ICANON;
    
    /*
     initialize all control characters
     default values can be found in /usr/include/termios.h, and are given
     in the comments, but we don't need them here
     */
    newtio.c_cc[VINTR]    = 0;     /* Ctrl-c */
    newtio.c_cc[VQUIT]    = 0;     /* Ctrl-\ */
    newtio.c_cc[VERASE]   = 0;     /* del */
    newtio.c_cc[VKILL]    = 0;     /* @ */
    newtio.c_cc[VEOF]     = 1;     /* Ctrl-d */
    newtio.c_cc[VTIME]    = 0;     /* inter-character timer unused */
    newtio.c_cc[VMIN]     = 3;     /* blocking read until 1 character arrives */
    newtio.c_cc[VSWTC]    = 0;     /* '\0' */
    newtio.c_cc[VSTART]   = 0;     /* Ctrl-q */
    newtio.c_cc[VSTOP]    = 0;     /* Ctrl-s */
    newtio.c_cc[VSUSP]    = 0;     /* Ctrl-z */
    newtio.c_cc[VEOL]     = 0;     /* '\0' */
    newtio.c_cc[VREPRINT] = 0;     /* Ctrl-r */
    newtio.c_cc[VDISCARD] = 0;     /* Ctrl-u */
    newtio.c_cc[VWERASE]  = 0;     /* Ctrl-w */
    newtio.c_cc[VLNEXT]   = 0;     /* Ctrl-v */
    newtio.c_cc[VEOL2]    = 0;     /* '\0' */
    
    /*
     now clean the modem line and activate the settings for the port
     */
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd, TCSANOW, &newtio);
    
}


void init_socket(int port){
    file_descriptor=socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);	//	Allocate a TCP socket for the internet.
    if	(file_descriptor<0)	// Check to see if there was a failure in allocation.
    {
	perror("Server: socket()");
	return;
    }
    //printf("Created socket on port %d\n", port);
    memset((void*)&server_address, 0, sizeof(server_address));	//	Clear the structure used to store the port and address that we want to listen on.
    server_address.sin_port=htons(port);	// Put port we want to listen on here. Take care to convert from host to network endian!!!
    server_address.sin_family=AF_INET;	// Don't ask.
    server_address.sin_addr.s_addr=htonl(INADDR_ANY);	// Address of the local server goes here. We put INADDR_ANY constant here to say we want to listen on all addresses on hosts with multiple ip addresses. Take care to convert from host endian to network endian!!!!
    if	(bind(file_descriptor, (struct sockaddr*)&server_address, sizeof(server_address))<0)	// Ok. We setup our structure. Now we tell the OS to do it.
    {
	perror("Server: bind()");
	return;
    }
    if	(listen(file_descriptor, 5)<0)	// Tell the OS to start listening on our port and local address for clients that want to connect.
    {
	perror("Server: listen()");
	return;
    }
    memset((void*)&inbound_address, 0, sizeof(inbound_address));
    inbound_address.sin_family=AF_INET;
    inbound_address_size=sizeof(inbound_address);	// Make sure you do this, or the inbound_address will not be filled with information about the incomming address.
    printf("Listening on port %d\n", port);
    
    
}

void server(int sig) {
	
   if(sig == SIGPIPE) {printf("Connection closed by client.\nWaiting for new connection...\n");
	
    }
    int i = 0;
    inbound_connection=accept(file_descriptor, (struct sockaddr*)&inbound_address, &inbound_address_size);	// Grab the first socket that represents the client that has connected. If none yet, block and wait till somebody does.
    if	(inbound_connection<0) {
	perror("accept()");
	return;
    }
    address_holder=(unsigned char*)&inbound_address.sin_addr.s_addr; // Save ourselves a call to the OS to convert.
    printf("AP-Module: Got a connection from %d.%d.%d.%d\n", address_holder[0], address_holder[1], address_holder[2], address_holder[3]);
    printf("Sending Serial Data...\n\n");
    
    // LOOP
    while(1){
	i = !i;
	do {
    	if (i)
	   write(fd, "X\r\n",3); // X and Y are not sent from RedBoot when bootup
        else 
	   write(fd, "Y\r\n",3);	
	res = read(fd, val, 200);
	val[res]=0;             /* set end of string, so we can printf */
	}while (atof(val)<0.1);

	//printf("%d %s",res, message);
	if (i){
	   message[0] = 'T';
	   message[1] = ' ';
	   message[2] = 0;
	   sleep(1);
	}
	else {
	   message[0] = 'H';
	   message[1] = ' ';
	   message[2] = 0;
	   sleep(1);	
	}
//	   message[0] = 0;
	strcat (message, val);

	fprintf(stdout,"\e[1BSent %ld lines of data.\n", sent_lines++);
	//parser(message);
	if	(write(inbound_connection, message, strlen(message))<0)	// Write the message to the client.
	{
	    perror("Server: write()");
	    return;
	}
    }
    
    
}


/*
 * Do a graceful exit when client closes socket or Ctrl-C is pressed.
 */


void quit(int sig){
    
    if(sig == SIGINT) {printf("Terminated by user (SIGINT).\n");}
    printf("Closing serial port and socket.\n");
    
    /* restore the old port settings */
    tcsetattr(fd, TCSANOW, &oldtio);
    /* close port */
    close(fd);
    
    close(inbound_connection);	// Tell the OS to clean up and free resources that we have used.
    close(file_descriptor);
    exit(0);
    
}



int main(int argc, char** argv) {
    
    // set default values
    char* modemdevice = MODEMDEVICE;
    int baudrate = BAUDRATE;
    int port = PORT;
    
    signal(SIGINT, quit);
    signal(SIGPIPE, server);
    
    printf("\e[1;32mCapBoard Serial-to-TCP bridge\e[0m\n");
        
    if (argc > 1){ // we have arguments
    
	// show usage info
	if (argc > 4 || !(strcmp(argv[1], "--help") && strcmp(argv[1], "-h"))) {
	    printf("Usage: %s [port] [devicefile] [baudrate]\n",argv[0]);
	    printf("Default: %s 2000 /dev/ttyS0 115200\n",argv[0]);
	    exit(0);
	}
	
	// parse arguments
	if (strstr(argv[1], "/") != NULL) {  // no port defined
	    modemdevice = argv[1]; // yes, a little memory leak I think
	    if (argc == 3) baudrate = atoi(argv[2]); // baudrate defined
	} else { // port defined
	    port = atoi(argv[1]);
	    if (argc > 2) modemdevice = argv[2]; // yes, a little memory leak I think
	    if (argc == 4 ) baudrate = atoi(argv[3]); // baudrate defined
	}
    }
  
    init_tty(modemdevice, baudrate);
    init_socket(port);
    server(SIGPIPE); // start the rumble
    
    return (EXIT_SUCCESS);
}

