/* 
 * The reboot_module listens the port 2001
 * for reboot signal from the server module
 * and reboots the AP.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>	// Include these for socket(), connect(), bind(), etc.
#include <sys/types.h>
#include <netdb.h>		// Include this for getprotobyname()
#include <string.h>	// Include this for memset()
#include <netinet/in.h>	// Include this for htonl(), htons(), etc.
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <signal.h>
#include <errno.h>



#define _POSIX_SOURCE 1 /* POSIX compliant source */
#define	PORT	2001  // Port to listen on...

void quit(int sig); // signal handler
void parser(char*);

char	val[200];
char	message[255];	// string to send to the client.
long	sent_lines = 0;



// Variables for the TCP socket component of the application.
int file_descriptor;	// File descriptor that represents the server socket.
int	inbound_connection;	// File descriptor that represents the socket of the inbound connection.
struct	sockaddr_in	server_address;	// Really only contains the port we want to listen on.
struct	sockaddr_in	inbound_address;	// Address of the inbound connection.
int	inbound_address_size;	// Size of the structure for the inbound connection.
unsigned	char	*address_holder;	// Pointer to simplify the extraction of IP addresses.
char	buffer[256];		// Buffer to hold incoming data from the client.

struct values{
	double val;
	int type;
};

void parser(char* value)
{
	struct values measure;
	
	if(value[0] == 'T' || value[0] == 't' )
	{
		measure.type = 1;
		measure.val = atof(value+2);
		printf("%lf\n", measure.val);
	}
	else if(value[0] == 'H' || value[0] == 'h')
	{
		measure.type = 2;
		measure.val = atof(value+2);
		printf("%lf\n", measure.val);
	}
	else
	{
		printf("Wrong Input\n");
	}
}


void init_socket(int port){
    file_descriptor=socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);	//	Allocate a TCP socket for the internet.
    if	(file_descriptor<0)	// Check to see if there was a failure in allocation.
    {
	perror("Server: socket()");
	return;
    }
    //printf("Created socket on port %d\n", port);
    memset((void*)&server_address, 0, sizeof(server_address));	//	Clear the structure used to store the port and address that we want to listen on.
    server_address.sin_port=htons(port);	// Put port we want to listen on here. Take care to convert from host to network endian!!!
    server_address.sin_family=AF_INET;	// Don't ask.
    server_address.sin_addr.s_addr=htonl(INADDR_ANY);	// Address of the local server goes here. We put INADDR_ANY constant here to say we want to listen on all addresses on hosts with multiple ip addresses. Take care to convert from host endian to network endian!!!!
    if	(bind(file_descriptor, (struct sockaddr*)&server_address, sizeof(server_address))<0)	// Ok. We setup our structure. Now we tell the OS to do it.
    {
	perror("Server: bind()");
	return;
    }
    if	(listen(file_descriptor, 5)<0)	// Tell the OS to start listening on our port and local address for clients that want to connect.
    {
	perror("Server: listen()");
	return;
    }
    memset((void*)&inbound_address, 0, sizeof(inbound_address));
    inbound_address.sin_family=AF_INET;
    inbound_address_size=sizeof(inbound_address);	// Make sure you do this, or the inbound_address will not be filled with information about the incomming address.
    printf("Listening on port %d\n", port);
    
    
}

void server(int sig) {
	
   if(sig == SIGPIPE) {printf("Connection closed by client.\nWaiting for new connection...\n");
	
    }
    //int i = 0;
    inbound_connection=accept(file_descriptor, (struct sockaddr*)&inbound_address, &inbound_address_size);	// Grab the first socket that represents the client that has connected. If none yet, block and wait till somebody does.
    if	(inbound_connection<0) {
	perror("accept()");
	return;
    }
    address_holder=(unsigned char*)&inbound_address.sin_addr.s_addr; // Save ourselves a call to the OS to convert.
    printf("Request from: %d.%d.%d.%d\n", address_holder[0], address_holder[1], address_holder[2], address_holder[3]);
    
    
    // LOOP
    while(1){
        int bytes = 0;

         if ((bytes = recv(inbound_connection, buffer, 255 , 0)) < 1) {
                perror("Failed to receive bytes from client!");
                exit(1);
              }


	    if (atoi(buffer) == 12345)
            break;
	
	}

	fprintf(stdout,"Rebooting..\n");
    int pid;
    //switch(pid=fork())
    //{
    //    case -1:
    //    perror("fork error 1"); break;
    //    case 0:
    //    execv("/bin/ls",NULL);
    //    perror("exec call 1 "); break;
    //}
    system ("reboot");

    printf("here..\n");


	//parser(message);
    
    
}


/*
 * Do a graceful exit when client closes socket or Ctrl-C is pressed.
 */


void quit(int sig){
    
    if(sig == SIGINT) {printf("Terminated by user (SIGINT).\n");}
    
    close(inbound_connection);	// Tell the OS to clean up and free resources that we have used.
    close(file_descriptor);
    exit(0);
    
}



int main(int argc, char** argv) {
    
    // set default values
    int port = PORT;
    
    signal(SIGINT, quit);
    signal(SIGPIPE, server);
    
    printf("\e[1;32mCapBoard Serial-to-TCP bridge\e[0m\n");
  
    init_socket(port);
    server(SIGPIPE); // start the rumble
    
    return (EXIT_SUCCESS);
}

